/*
 * tft280_ili9341.c
 *
 *  Created on: 20231219
 *      Author: www54
 */
/*********************************************************************************************************************
* ߶壺
*                   ------------------------------------
*                   ģܽ             Ƭܽ
*                   SCL                  鿴 zf_device_tft280.h  TFT280_SCL_PIN 궨
*                   SDA                  鿴 zf_device_tft280.h  TFT280_SDA_PIN 궨
*                   RES                  鿴 zf_device_tft280.h  TFT280_RES_PIN 궨
*                   DC                   鿴 zf_device_tft280.h  TFT280_DC_PIN 궨
*                   CS                   鿴 zf_device_tft280.h  TFT280_CS_PIN 궨
*                   BL                   鿴 zf_device_tft280.h  TFT280_BL_PIN 궨
*                   VCC                 3.3VԴ
*                   GND                 Դ
*                   ֱ320*240
*                   ------------------------------------
********************************************************************************************************************/

#include "zf_common_clock.h"
#include "zf_common_debug.h"
#include "zf_common_font.h"
#include "zf_common_function.h"
#include "zf_driver_delay.h"
#include "zf_driver_soft_spi.h"
#include "zf_driver_spi.h"

#include "tft280_st7789v.h"

static uint16                   tft280_pencolor     = TFT280_DEFAULT_PENCOLOR;
static uint16                   tft280_bgcolor      = TFT280_DEFAULT_BGCOLOR;

static tft280_dir_enum          tft280_display_dir  = TFT280_DEFAULT_DISPLAY_DIR;
tft280_font_size_enum    tft280_display_font = TFT280_DEFAULT_DISPLAY_FONT;

static uint16            tft280_x_max        = TFT280_X_MAX;
static uint16            tft280_y_max        = TFT280_Y_MAX;

uint16 pix_color = 0;

#if TFT280_USE_SOFT_SPI
static soft_spi_info_struct             tft280_spi;
//-------------------------------------------------------------------------------------------------------------------
//      TFT280 SPI д 8bit 
// ˵     data            
// ز     void
// ʹʾ     tft280_write_8bit_data(dat);
// עϢ     ڲ
//-------------------------------------------------------------------------------------------------------------------
#define tft280_write_8bit_data(data)    (soft_spi_write_8bit(&tft280_spi, (data)))

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 SPI д 16bit 
// ˵     data            
// ز     void
// ʹʾ     tft280_write_16bit_data(x1 + 52);
// עϢ     ڲ
//-------------------------------------------------------------------------------------------------------------------
#define tft280_write_16bit_data(data)   (soft_spi_write_16bit(&tft280_spi, (data)))
#else
//-------------------------------------------------------------------------------------------------------------------
//      TFT280 SPI д 8bit 
// ˵     data            
// ز     void
// ʹʾ     tft280_write_8bit_data(dat);
// עϢ     ڲ
//-------------------------------------------------------------------------------------------------------------------
#define tft280_write_8bit_data(data)    (spi_write_8bit(TFT280_SPI, (data)))

//-------------------------------------------------------------------------------------------------------------------
//      IPS114 SPI д 16bit 
// ˵     data            
// ز     void
// ʹʾ     ips114_write_16bit_data(x1 + 52);
// עϢ     ڲ
//-------------------------------------------------------------------------------------------------------------------
#define tft280_write_16bit_data(data)   (spi_write_16bit(TFT280_SPI, (data)))
#endif

//-------------------------------------------------------------------------------------------------------------------
//      д
// ˵     dat             
// ز     void
// ʹʾ     tft280_write_index(0x2a);
// עϢ     ڲ
//-------------------------------------------------------------------------------------------------------------------
static void tft280_write_index (const uint8 dat)
{
    TFT280_DC(0);
    tft280_write_8bit_data(dat);
    TFT280_DC(1);
}

//-------------------------------------------------------------------------------------------------------------------
//      ʾ ڲ
// ˵     x1              ʼx
// ˵     y1              ʼy
// ˵     x2              x
// ˵     y2              y
// ز     void
// ʹʾ     tft280_set_region(0, 0, tft280_x_max - 1, tft280_y_max - 1);
// עϢ     ڲ
//-------------------------------------------------------------------------------------------------------------------
void tft280_set_region (uint16 x1, uint16 y1, uint16 x2, uint16 y2)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    // һʾõĺ ԼһﳬĻʾΧ
    zf_assert(x1 < tft280_x_max);
    zf_assert(y1 < tft280_y_max);
    zf_assert(x2 < tft280_x_max);
    zf_assert(y2 < tft280_y_max);

    switch(tft280_display_dir)
    {
        case TFT280_PORTAIT:
        case TFT280_PORTAIT_180:
        {
            tft280_write_index(0x2a);
            tft280_write_8bit_data(x1>>8);
            tft280_write_8bit_data(x1);
            tft280_write_8bit_data(x2>>8);
            tft280_write_8bit_data(x2);

            tft280_write_index(0x2b);
            tft280_write_8bit_data(y1>>8);
            tft280_write_8bit_data(y1);
            tft280_write_8bit_data(y2>>8);
            tft280_write_8bit_data(y2);
        }break;
        case TFT280_CROSSWISE:
        case TFT280_CROSSWISE_180:
        {
            tft280_write_index(0x2a);
            tft280_write_8bit_data(x1>>8);
            tft280_write_8bit_data(x1);
            tft280_write_8bit_data(x2>>8);
            tft280_write_8bit_data(x2);

            tft280_write_index(0x2b);
            tft280_write_8bit_data(y1>>8);
            tft280_write_8bit_data(y1);
            tft280_write_8bit_data(y2>>8);
            tft280_write_8bit_data(y2);
        }break;
    }
    tft280_write_index(0x2c);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾDEBUGϢʼ
// ˵     void
// ز     void
// ʹʾ     tft280_debug_init();
// עϢ     ڲʹ
//-------------------------------------------------------------------------------------------------------------------
static void tft280_debug_init (void)
{
    debug_output_struct info;
    debug_output_struct_init(&info);

    info.type_index = 1;
    info.display_x_max = tft280_x_max;
    info.display_y_max = tft280_y_max;

    switch(tft280_display_font)
    {
        case TFT280_6X8_FONT:
        {
            info.font_x_size = 6;
            info.font_y_size = 8;
        }break;
        case TFT280_8X16_FONT:
        {
            info.font_x_size = 8;
            info.font_y_size = 16;
        }break;
        case TFT280_16X16_FONT:
        {
            // ݲ֧
        }break;
    }
    info.output_screen = tft280_show_string;
    info.output_screen_clear = tft280_clear;

    debug_output_init(&info);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 
// ˵     void
// ز     void
// ʹʾ     tft280_clear();
// עϢ     Ļճɱɫ
//-------------------------------------------------------------------------------------------------------------------
void tft280_clear (void)
{
    uint32 i = tft280_x_max * tft280_y_max;

    TFT280_CS(0);
    tft280_set_region(0, 0, tft280_x_max - 1, tft280_y_max - 1);
    for( ; 0 < i; i --)
    {
        tft280_write_16bit_data(tft280_bgcolor);
    }
    TFT280_CS(1);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 
// ˵     color           ɫʽ RGB565 ߿ʹ zf_common_font.h ڳɫ궨
// ز     void
// ʹʾ     tft280_full(RGB565_YELLOW);
// עϢ
//-------------------------------------------------------------------------------------------------------------------
void tft280_full (const uint16 color)
{
    uint32 i = tft280_x_max * tft280_y_max;

    TFT280_CS(0);
    tft280_set_region(0, 0, tft280_x_max - 1, tft280_y_max - 1);
    for( ; 0 < i; i --)
    {
        tft280_write_16bit_data(color);
    }
    TFT280_CS(1);
}

/**
 * @brief 
 * @param color ɫ
 * @param x1 ʼx
 * @param y1 ʼy
 * @param x2 x
 * @param y2 y
 */
void tft280_full_area (const uint16 color, uint16 x1, uint16 y1, uint16 x2, uint16 y2)
{
    uint32 i = (x2 - x1 + 1) * (y2 - y1 + 1);

    TFT280_CS(0);
    tft280_set_region(x1, y1, x2, y2);
    for( ; 0 < i; i --)
    {
        tft280_write_16bit_data(color);
    }
    TFT280_CS(1);
}

//-------------------------------------------------------------------------------------------------------------------
//      ʾ
// ˵     dir             ʾ   zf_device_ips114.h  tft280_dir_enum ö嶨
// ز     void
// ʹʾ     tft280_set_dir(TFT280_CROSSWISE);
// עϢ     ֻڳʼĻ֮ǰòЧ
//-------------------------------------------------------------------------------------------------------------------
void tft280_set_dir (tft280_dir_enum dir)
{
    tft280_display_dir = dir;
    switch(tft280_display_dir)
    {
        case TFT280_PORTAIT:
        case TFT280_PORTAIT_180:
        {
            tft280_x_max = TFT280_Y_MAX;
            tft280_y_max = TFT280_X_MAX;
        }break;
        case TFT280_CROSSWISE:
        case TFT280_CROSSWISE_180:
        {
            tft280_x_max = TFT280_X_MAX;
            tft280_y_max = TFT280_Y_MAX;
        }break;
    }
}

//-------------------------------------------------------------------------------------------------------------------
//      ʾ
// ˵     dir             ʾ   zf_device_tft280.h  tft280_font_size_enum ö嶨
// ز     void
// ʹʾ     tft280_set_font(TFT280_8x16_FONT);
// עϢ     ʱ úЧ ʾµС
//-------------------------------------------------------------------------------------------------------------------
void tft280_set_font (tft280_font_size_enum font)
{
    tft280_display_font = font;
}

//-------------------------------------------------------------------------------------------------------------------
//      ʾɫ
// ˵     pen             ɫʽ RGB565 ߿ʹ zf_common_font.h ڳɫ궨
// ˵     bgcolor         ɫʽ RGB565 ߿ʹ zf_common_font.h ڳɫ궨
// ز     void
// ʹʾ     tft280_set_color(RGB565_WHITE, RGB565_BLACK);
// עϢ     ɫͱɫҲʱ úЧ
//-------------------------------------------------------------------------------------------------------------------
void tft280_set_color (uint16 pen, const uint16 bgcolor)
{
    tft280_pencolor = pen;
    tft280_bgcolor = bgcolor;
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     dat             Ҫʾɫ
// ز     void
// ʹʾ     tft280_draw_point(0, 0, RGB565_RED);            //  0,0 һɫĵ
// עϢ
//-------------------------------------------------------------------------------------------------------------------
void tft280_draw_point (uint16 x, uint16 y, const uint16 color)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);

    TFT280_CS(0);
    tft280_set_region(x, y, x, y);
    tft280_write_16bit_data(color);
    TFT280_CS(1);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 
// ˵     x_start         x
// ˵     y_start         y
// ˵     x_end           xյ
// ˵     y_end           yյ
// ˵     dat             Ҫʾɫ
// ز     void
// ʹʾ     tft280_draw_line(0, 0, 10, 10,RGB565_RED);      //  0,0  10,10 һɫ
// עϢ
//-------------------------------------------------------------------------------------------------------------------
void tft280_draw_line (uint16 x_start, uint16 y_start, uint16 x_end, uint16 y_end, const uint16 color)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x_start < tft280_x_max);
    zf_assert(y_start < tft280_y_max);
    zf_assert(x_end < tft280_x_max);
    zf_assert(y_end < tft280_y_max);

    int16 x_dir = (x_start < x_end ? 1 : -1);
    int16 y_dir = (y_start < y_end ? 1 : -1);
    float temp_rate = 0;
    float temp_b = 0;

    do
    {
        if(x_start != x_end)
        {
            temp_rate = (float)(y_start - y_end) / (float)(x_start - x_end);
            temp_b = (float)y_start - (float)x_start * temp_rate;
        }
        else
        {
            while(y_start != y_end)
            {
                tft280_draw_point(x_start, y_start, color);
                y_start += y_dir;
            }
            break;
        }

        if(func_abs(y_start - y_end) > func_abs(x_start - x_end))
        {
            while(y_start != y_end)
            {
                tft280_draw_point(x_start, y_start, color);
                y_start += y_dir;
                x_start = (int16)(((float)y_start - temp_b) / temp_rate);
            }
        }
        else
        {
            while(x_start != x_end)
            {
                tft280_draw_point(x_start, y_start, color);
                x_start += x_dir;
                y_start = (int16)((float)x_start * temp_rate + temp_b);
            }
        }
    }while(0);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾַ
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     dat             Ҫʾַ
// ز     void
// ʹʾ     tft280_show_char(0, 0, 'x');                    //  0,0 дһַ x
// עϢ
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_char (uint16 x, uint16 y, const char dat)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);

    uint8 i = 0, j = 0;

    TFT280_CS(0);
    switch(tft280_display_font)
    {
        case TFT280_6X8_FONT:
        {
            for(i = 0; 6 > i; i ++)
            {
                tft280_set_region(x + i, y, x + i, y + 8);
                //  32 ΪȡģǴӿոʼȡ ո ascii  32
                uint8 temp_top = ascii_font_6x8[dat - 32][i];
                for(j = 0; 8 > j; j ++)
                {
                    if(temp_top & 0x01)
                    {
                        tft280_write_16bit_data(tft280_pencolor);
                    }
                    else
                    {
                        tft280_write_16bit_data(tft280_bgcolor);
                    }
                    temp_top >>= 1;
                }
            }
        }break;
        case TFT280_8X16_FONT:
        {
            for(i = 0; 8 > i; i ++)
            {
                tft280_set_region(x + i, y, x + i, y + 15);
                //  32 ΪȡģǴӿոʼȡ ո ascii  32
                uint8 temp_top = ascii_font_8x16[dat - 32][i];
                uint8 temp_bottom = ascii_font_8x16[dat - 32][i + 8];
                for(j = 0; 8 > j; j ++)
                {
                    if(temp_top & 0x01)
                    {
                        tft280_write_16bit_data(tft280_pencolor);
                    }
                    else
                    {
                        tft280_write_16bit_data(tft280_bgcolor);
                    }
                    temp_top >>= 1;
                }
                for(j = 0; 8 > j; j ++)
                {
                    if(temp_bottom & 0x01)
                    {
                        tft280_write_16bit_data(tft280_pencolor);
                    }
                    else
                    {
                        tft280_write_16bit_data(tft280_bgcolor);
                    }
                    temp_bottom >>= 1;
                }
            }
        }break;
        case TFT280_16X16_FONT:
        {
            // ݲ֧
        }break;
    }
    TFT280_CS(1);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾַ
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     dat             Ҫʾַ
// ز     void
// ʹʾ     tft280_show_string(0, 0, "seekfree");
// עϢ
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_string (uint16 x, uint16 y, const char dat[])
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);

    uint16 j = 0;
    while('\0' != dat[j])
    {
        switch(tft280_display_font)
        {
            case TFT280_6X8_FONT:   tft280_show_char(x + 6 * j, y, dat[j]); break;
            case TFT280_8X16_FONT:  tft280_show_char(x + 8 * j, y, dat[j]); break;
            case TFT280_16X16_FONT: break;                                      // ݲ֧
        }
        j ++;
    }
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾ32λз (ȥЧ0)
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     dat             Ҫʾı  int32
// ˵     num             Ҫʾλ 10λ  
// ز     void
// ʹʾ     tft280_show_int(0, 0, x, 3);                    // x Ϊ int32 int16 int8 
// עϢ     ʾһ -
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_int (uint16 x, uint16 y, const int32 dat, uint8 num)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(0 < num);
    zf_assert(10 >= num);

    int32 dat_temp = dat;
    int32 offset = 1;
    char data_buffer[12];

    memset(data_buffer, 0, 12);
    memset(data_buffer, ' ', num + 1);

    // ʾ 123 ʾ 2 λӦʾ 23
    if(10 > num)
    {
        for(; 0 < num; num --)
        {
            offset *= 10;
        }
        dat_temp %= offset;
    }
    func_int_to_str(data_buffer, dat_temp);
    tft280_show_string(x, y, (const char *)&data_buffer);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾ32λ޷ (ȥЧ0)
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     dat             Ҫʾı  uint32
// ˵     num             Ҫʾλ 10λ  
// ز     void
// ʹʾ     tft280_show_uint(0, 0, x, 3);                   // x Ϊ uint32 uint16 uint8 
// עϢ
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_uint (uint16 x, uint16 y, const uint32 dat, uint8 num)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(0 < num);
    zf_assert(10 >= num);

    uint32 dat_temp = dat;
    int32 offset = 1;
    char data_buffer[12];
    memset(data_buffer, 0, 12);
    memset(data_buffer, ' ', num);

    // ʾ 123 ʾ 2 λӦʾ 23
    if(10 > num)
    {
        for(; 0 < num; num --)
        {
            offset *= 10;
        }
        dat_temp %= offset;
    }
    func_uint_to_str(data_buffer, dat_temp);
    tft280_show_string(x, y, (const char *)&data_buffer);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾ(ȥЧ0)
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     dat             Ҫʾı  float
// ˵     num             λʾ   8λ
// ˵     pointnum        Сλʾ   6λ
// ز     void
// ʹʾ     tft280_show_float(0, 0, x, 2, 3);               // ʾ   ʾ2λ   Сʾλ
// עϢ     رע⵱Сʾֵдֵһʱ
//              ڸȶʧ⵼µģⲢʾ⣬
//              й飬аٶѧϰ   ȶʧ⡣
//              ʾһ -
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_float (uint16 x, uint16 y, const double dat, uint8 num, uint8 pointnum)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(0 < num);
    zf_assert(8 >= num);
    zf_assert(0 < pointnum);
    zf_assert(6 >= pointnum);

    double dat_temp = dat;
    double offset = 1.0;
    char data_buffer[17];
    memset(data_buffer, 0, 17);
    memset(data_buffer, ' ', num + pointnum + 2);

    // ʾ 123 ʾ 2 λӦʾ 23
    for(; 0 < num; num --)
    {
        offset *= 10;
    }
    dat_temp = dat_temp - ((int)dat_temp / (int)offset) * offset;
    func_double_to_str(data_buffer, dat_temp, pointnum);
    tft280_show_string(x, y, data_buffer);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾֵͼ ÿ˸һֽ
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     *image          ͼָ
// ˵     width           ͼʵʿ
// ˵     height          ͼʵʸ߶
// ˵     dis_width       ͼʾ Χ [0, tft280_x_max]
// ˵     dis_height      ͼʾ߶ Χ [0, tft280_y_max]
// ز     void
// ʹʾ     tft280_show_binary_image(0, 0, ov7725_image_binary[0], OV7725_W, OV7725_H, OV7725_W / 2, OV7725_H / 2);
// עϢ     ʾСδѹѹֵͼ
//              ֱʾδѹĶֵͼ
//              ֱʾδѹĶֵͼ
//              ֱʾδѹĶֵͼ
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_binary_image (uint16 x, uint16 y, const uint8 *image, uint16 width, uint16 height, uint16 dis_width, uint16 dis_height)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(NULL != image);

    uint32 i = 0, j = 0;
    uint8 temp = 0;
    uint32 width_index = 0, height_index = 0;

    TFT280_CS(0);
    tft280_set_region(x, y, x + dis_width - 1, y + dis_height - 1);             // ʾ

    for(j = 0; j < dis_height; j ++)
    {
        height_index = j * height / dis_height;
        for(i = 0; i < dis_width; i ++)
        {
            width_index = i * width / dis_width;
            temp = *(image + height_index * width / 8 + width_index / 8);       // ȡص
            if(0x80 & (temp << (width_index % 8)))
            {
                tft280_write_16bit_data(RGB565_WHITE);
            }
            else
            {
                tft280_write_16bit_data(RGB565_BLACK);
            }
        }
    }
    TFT280_CS(1);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾ 8bit Ҷͼ ֵֵ
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     *image          ͼָ
// ˵     width           ͼʵʿ
// ˵     height          ͼʵʸ߶
// ˵     dis_width       ͼʾ Χ [0, tft280_x_max]
// ˵     dis_height      ͼʾ߶ Χ [0, tft280_y_max]
// ˵     threshold       ֵʾֵ 0-ֵ
// ز     void
// ʹʾ     tft280_show_gray_image(0, 0, mt9v03x_image[0], MT9V03X_W, MT9V03X_H, MT9V03X_W / 2, MT9V03X_H / 2, 0);
// עϢ     ʾͼ
//              Ҫʾֵͼ ֱ޸һΪҪĶֵֵ
//              Ҫʾֵͼ ֱ޸һΪҪĶֵֵ
//              Ҫʾֵͼ ֱ޸һΪҪĶֵֵ
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_gray_image (uint16 x, uint16 y, const uint8 *image, uint16 width, uint16 height, uint16 dis_width, uint16 dis_height, uint8 threshold)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(NULL != image);

    uint32 i = 0, j = 0;
    uint16 color = 0,temp = 0;
    uint32 width_index = 0, height_index = 0;

    TFT280_CS(0);
    tft280_set_region(x, y, x + dis_width - 1, y + dis_height - 1);             // ʾ

    for(j = 0; j < dis_height; j ++)
    {
        height_index = j * height / dis_height;
        for(i = 0; i < dis_width; i ++)
        {
            width_index = i * width / dis_width;
            temp = *(image + height_index * width + width_index);               // ȡص
            if(threshold == 0)
            {
                color = (0x001f & ((temp) >> 3)) << 11;
                color = color | (((0x003f) & ((temp) >> 2)) << 5);
                color = color | (0x001f & ((temp) >> 3));
                tft280_write_16bit_data(color);
            }
            else if(temp < threshold)
            {
                tft280_write_16bit_data(RGB565_BLACK);
            }
            else
            {
                tft280_write_16bit_data(RGB565_WHITE);
            }
        }
    }
    TFT280_CS(1);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾ RGB565 ɫͼ
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     *image          ͼָ
// ˵     width           ͼʵʿ
// ˵     height          ͼʵʸ߶
// ˵     dis_width       ͼʾ Χ [0, tft280_x_max]
// ˵     dis_height      ͼʾ߶ Χ [0, tft280_y_max]
// ˵     color_mode      ɫģʽ 0-λǰ 1-λǰ
// ز     void
// ʹʾ     tft280_show_rgb565_image(0, 0, scc8660_image[0], SCC8660_W, SCC8660_H, SCC8660_W / 2, SCC8660_H / 2, 1);
// עϢ     ʾͫ RGB565 ͼ
//              Ҫʾλǰ RGB565 ͼ ޸һ
//              Ҫʾλǰ RGB565 ͼ ޸һ
//              Ҫʾλǰ RGB565 ͼ ޸һ
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_rgb565_image (uint16 x, uint16 y, const uint16 *image, uint16 width, uint16 height, uint16 dis_width, uint16 dis_height, uint8 color_mode)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(NULL != image);

    uint32 i = 0, j = 0;
    uint16 color = 0;
    uint32 width_index = 0, height_index = 0;

    TFT280_CS(0);
    tft280_set_region(x, y, x + dis_width - 1, y + dis_height - 1);             // ʾ

    for(j = 0; j < dis_height; j ++)
    {
        height_index = j * height / dis_height;
        for(i = 0; i < dis_width; i ++)
        {
            width_index = i * width / dis_width;
            color = *(image + height_index * width + width_index);              // ȡص
            if(color_mode)
            {
                color = ((color & 0xff) << 8) | (color >> 8);
            }
            tft280_write_16bit_data(color);
        }
    }
    TFT280_CS(1);
}

//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʾ
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     *wave           ָ
// ˵     width           ʵʿ
// ˵     value_max       ʵֵ
// ˵     dis_width       ʾ Χ [0, tft280_x_max]
// ˵     dis_value_max   ʾֵ Χ [0, tft280_y_max]
// ز     void
// ʹʾ     tft280_show_wave(32, 64, data, 128, 64, 64, 32);
// עϢ
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_wave (uint16 x, uint16 y, const uint16 *wave, uint16 width, uint16 value_max, uint16 dis_width, uint16 dis_value_max)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(NULL != wave);

    uint32 i = 0, j = 0;
    uint32 width_index = 0, value_max_index = 0;

    TFT280_CS(0);
    tft280_set_region(x, y, x + dis_width - 1, y + dis_value_max - 1);          // ʾ
    for(i = 0; i < dis_value_max; i ++)
    {
        for(j = 0; j < dis_width; j ++)
        {
            tft280_write_16bit_data(tft280_bgcolor);
        }
    }
    TFT280_CS(1);

    for(i = 0; i < dis_width; i ++)
    {
        width_index = i * width / dis_width;
        value_max_index = *(wave + width_index) * (dis_value_max - 1) / value_max;
        tft280_draw_point(i + x, (dis_value_max - 1) - value_max_index + y, tft280_pencolor);
    }
}

//-------------------------------------------------------------------------------------------------------------------
//      ʾ
// ˵     x               x Χ [0, tft280_x_max-1]
// ˵     y               y Χ [0, tft280_y_max-1]
// ˵     size            ȡģʱõĺС Ҳһռõĵ󳤿Ϊٸ ȡģʱҪһ
// ˵     *chinese_buffer Ҫʾĺ
// ˵     number          Ҫʾλ
// ˵     color           ʾɫ
// ز     void
// ʹʾ     tft280_show_chinese(0, 0, 16, chinese_test[0], 4, RGB565_RED);//ʾfontļ ʾ
// עϢ     ʹPCtoLCD2002ȡģ           롢ʽ˳   16*16
//-------------------------------------------------------------------------------------------------------------------
void tft280_show_chinese (uint16 x, uint16 y, uint8 size, const uint8 *chinese_buffer, uint8 number, const uint16 color)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(NULL != chinese_buffer);

    int i = 0, j = 0, k = 0;
    uint8 temp = 0, temp1 = 0, temp2 = 0;
    const uint8 *p_data = chinese_buffer;

    temp2 = size / 8;

    TFT280_CS(0);
    tft280_set_region(x, y, number * size - 1 + x, y + size - 1);

    for(i = 0; i < size; i ++)
    {
        temp1 = number;
        p_data = chinese_buffer + i * temp2;
        while(temp1 --)
        {
            for(k = 0; k < temp2; k ++)
            {
                for(j = 8; 0 < j; j --)
                {
                    temp = (*p_data >> (j - 1)) & 0x01;
                    if(temp)
                    {
                        tft280_write_16bit_data(color);
                    }
                    else
                    {
                        tft280_write_16bit_data(tft280_bgcolor);
                    }
                }
                p_data ++;
            }
            p_data = p_data - temp2 + temp2 * size;
        }
    }
    TFT280_CS(1);
}
/*********************************************************************
 * @fn      DMA_Tx_Init
 *
 * @brief   Initializes the DMAy Channelx configuration.
 *
 * @param   DMA_CHx - x can be 1 to 7.
 *          ppadr - Peripheral base address.
 *          memadr - Memory base address.
 *          bufsize - DMA channel buffer size.
 *
 * @return  none
 */
DMA_InitTypeDef DMA_InitStructure={0};
static void SPI_DMA_Tx_Init( DMA_Channel_TypeDef* DMA_CHx, u32 ppadr, u32 memadr, u16 bufsize)
{
    switch(TFT280_SPI){
        case SPI_1:
        case SPI_2:RCC_AHBPeriphClockCmd( RCC_AHBPeriph_DMA1, ENABLE );break;
        case SPI_3:RCC_AHBPeriphClockCmd( RCC_AHBPeriph_DMA2, ENABLE );break;
    }

    DMA_DeInit(DMA_CHx);

    DMA_InitStructure.DMA_PeripheralBaseAddr = ppadr;
    DMA_InitStructure.DMA_MemoryBaseAddr = memadr;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = bufsize;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

    DMA_Init( DMA_CHx, &DMA_InitStructure );

    DMA_Cmd( TFT280_SPI_DMA_TX_CH, ENABLE );
}
void switch_spi_dma_data_addr(void)
{
    DMA_DeInit(TFT280_SPI_DMA_TX_CH);
    DMA_InitStructure.DMA_MemoryBaseAddr = image_busy_flag[0][0];
    DMA_Init( TFT280_SPI_DMA_TX_CH, &DMA_InitStructure );
}
void tft280_display_image(uint16 x, uint16 y, const uint8 *image, uint16 width, uint16 height, uint16 dis_width, uint16 dis_height, uint8 threshold)
{
    // ˶Ϣ ʾλ
    // ôһĻʾʱ򳬹ĻֱʷΧ
    zf_assert(x < tft280_x_max);
    zf_assert(y < tft280_y_max);
    zf_assert(NULL != image);

    uint32 i = 0, j = 0;
    uint16 temp = 0;
    uint32 width_index = 0, height_index = 0;

    TFT280_CS(0);
    tft280_set_region(x, y, x + dis_width - 1, y + dis_height - 1);             // ʾ

    for(j = 0; j < dis_height; j ++)
    {
        height_index = j * height / dis_height;
        for(i = 0; i < dis_width; i ++)
        {
            width_index = i * width / dis_width;
            temp = *(image + height_index * width + width_index);               // ȡص
            if(threshold == 0)
            {
                pix_color = (0x001f & ((temp) >> 3)) << 11;
                pix_color = pix_color | (((0x003f) & ((temp) >> 2)) << 5);
                pix_color = pix_color | (0x001f & ((temp) >> 3));
            }
            else if(temp < threshold)
            {
                pix_color = RGB565_BLACK;
            }
            else
            {
                pix_color = RGB565_WHITE;
            }
            SPI_I2S_DMACmd(((SPI_TypeDef *)(spi_index[TFT280_SPI])), SPI_I2S_DMAReq_Tx, ENABLE );
//            while(!DMA_GetFlagStatus(DMA2_FLAG_TC2));
//            DMA_ClearFlag(DMA2_FLAG_TC2);
        }
    }
    TFT280_CS(1);
}
//-------------------------------------------------------------------------------------------------------------------
//      TFT280 ʼ
// ز     void
// ز     void
// ʹʾ     tft280_init();
// עϢ
//-------------------------------------------------------------------------------------------------------------------
void tft280_init (void)
{
#if TFT280_USE_SOFT_SPI
    soft_spi_init(&tft280_spi, 0, TFT280_SOFT_SPI_DELAY, TFT280_SCL_PIN, TFT280_SDA_PIN, SOFT_SPI_PIN_NULL, SOFT_SPI_PIN_NULL);
#else
    spi_init(TFT280_SPI, SPI_MODE0, TFT280_SPI_SPEED, TFT280_SCL_PIN, TFT280_SDA_PIN, SPI_MISO_NULL, SPI_CS_NULL);
#endif
#if TFT280_SPI_DMA
    SPI_DMA_Tx_Init(TFT280_SPI_DMA_TX_CH, ((SPI_TypeDef *)(spi_index[TFT280_SPI]))->DATAR, (u32)TFT280_SPI_DMA_TX_DATA_ADDR, TFT280_SPI_DMA_BUFF_SIZE);
#endif
    gpio_init(TFT280_DC_PIN, GPO, GPIO_LOW, GPO_PUSH_PULL);
    gpio_init(TFT280_RES_PIN, GPO, GPIO_LOW, GPO_PUSH_PULL);
    gpio_init(TFT280_CS_PIN, GPO, GPIO_HIGH, GPO_PUSH_PULL);
    gpio_init(TFT280_BL_PIN, GPO, GPIO_HIGH, GPO_PUSH_PULL);

    tft280_set_dir(tft280_display_dir);
    tft280_set_color(tft280_pencolor, tft280_bgcolor);

    TFT280_RST(0);
    system_delay_ms(100);

    TFT280_RST(1);
    system_delay_ms(100);

    TFT280_CS(0);

    tft280_write_index(0x11);
    system_delay_ms(120);

    //------------------------------display and color format setting--------------------------------//
    tft280_write_index(0x36);
    switch(tft280_display_dir)                                                  // y x v
    {
        case TFT280_PORTAIT:        tft280_write_8bit_data(1<<7 | 1<<6 | 0<<5);  break;
        case TFT280_PORTAIT_180:    tft280_write_8bit_data(0<<7 | 0<<6 | 0<<5);  break;
        case TFT280_CROSSWISE:      tft280_write_8bit_data(1<<7 | 0<<6 | 1<<5);  break;
        case TFT280_CROSSWISE_180:  tft280_write_8bit_data(0<<7 | 1<<6 | 1<<5);  break;
    }
    tft280_write_index(0x3a);
    tft280_write_8bit_data(0x55);
    //--------------------------------ST7789V Frame rate setting----------------------------------//
    tft280_write_index(0xb2);
    tft280_write_8bit_data(0x0c);
    tft280_write_8bit_data(0x0c);
    tft280_write_8bit_data(0x00);
    tft280_write_8bit_data(0x33);
    tft280_write_8bit_data(0x33);
    tft280_write_index(0xb7);
    tft280_write_8bit_data(0x35);
    //---------------------------------ST7789V Power setting--------------------------------------//
    tft280_write_index(0xbb);
    tft280_write_8bit_data(0x28);
    tft280_write_index(0xc0);
    tft280_write_8bit_data(0x2c);
    tft280_write_index(0xc2);
    tft280_write_8bit_data(0x01);
    tft280_write_index(0xc3);
    tft280_write_8bit_data(0x0b);
    tft280_write_index(0xc4);
    tft280_write_8bit_data(0x20);
    tft280_write_index(0xc6);
    tft280_write_8bit_data(0x0f);
    tft280_write_index(0xd0);
    tft280_write_8bit_data(0xa4);
    tft280_write_8bit_data(0xa1);
    //--------------------------------ST7789V gamma setting---------------------------------------//
    tft280_write_index(0xe0);
    tft280_write_8bit_data(0xd0);
    tft280_write_8bit_data(0x01);
    tft280_write_8bit_data(0x08);
    tft280_write_8bit_data(0x0f);
    tft280_write_8bit_data(0x11);
    tft280_write_8bit_data(0x2a);
    tft280_write_8bit_data(0x36);
    tft280_write_8bit_data(0x55);
    tft280_write_8bit_data(0x44);
    tft280_write_8bit_data(0x3a);
    tft280_write_8bit_data(0x0b);
    tft280_write_8bit_data(0x06);
    tft280_write_8bit_data(0x11);
    tft280_write_8bit_data(0x20);

    tft280_write_index(0xe1);
    tft280_write_8bit_data(0xd0);
    tft280_write_8bit_data(0x02);
    tft280_write_8bit_data(0x07);
    tft280_write_8bit_data(0x0a);
    tft280_write_8bit_data(0x0b);
    tft280_write_8bit_data(0x18);
    tft280_write_8bit_data(0x34);
    tft280_write_8bit_data(0x43);
    tft280_write_8bit_data(0x4a);
    tft280_write_8bit_data(0x2b);
    tft280_write_8bit_data(0x1b);
    tft280_write_8bit_data(0x1c);
    tft280_write_8bit_data(0x22);
    tft280_write_8bit_data(0x1f);

    tft280_write_index(0x29);

    TFT280_CS(1);

    tft280_clear();
    tft280_debug_init();
}

